/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include <qtimer.h>

#include <kapplication.h>

#include "debug.h"
#include "kadu.h"
#include "message_box.h"
#include "userlist.h"
#include "config_dialog.h"
#include "status.h"
#include "modules.h"

#include "amarok.h"

#define MODULE_AMAROK_VERSION 1.13

// For CP1250->ISO8859-2 converter
const char CODE_CP1250[]    = {0xb9,0x9c,0x9f,0xa5,0x8c,0x8f};
const char CODE_ISO8859_2[] = {0xb1,0xb6,0xbc,0xa1,0xa6,0xac};

// For ID3 tags signatures cutter
const char DEFAULT_SIGNATURES[] = "! WWW.POLSKIE-MP3.TK ! \n! www.polskie-mp3.tk ! ";

AmaroK* amarok;

extern "C" int amarok_init()
{
    amarok = new AmaroK();
    return 0;
}

extern "C" void amarok_close()
{
    delete amarok;
}

AmaroK::AmaroK()
{
    kdebugf();

    // amaroK module objects
    client = kapp->dcopClient();
    client->attach();
    config = new ConfigFile(ggPath(QString("/kadu-amarok.conf")));
    timer = new QTimer();

    // Chat windows menu
    menu = new QPopupMenu();
    popups[0] = menu->insertItem(tr("Put formated string"), this, SLOT(putSongTitle(int)));
    popups[1] = menu->insertItem(tr("Put song title"), this, SLOT(putSongTitle(int)));

    // Statuses menu entry
    bool menuPos = config->readBoolEntry("amaroK", "dockMenu", false);
    if (menuPos)
        popups[2] = dockMenu->insertItem(tr("Enable amaroK statuses"), this, SLOT(toggleTimer(int)), 0, -1, 10);
    else
        popups[2] = kadu->mainMenu()->insertItem(tr("Enable amaroK statuses"), this, SLOT(toggleTimer(int)), 0, -1, 0);
 
    // Handle chat buttons
    connect(timer, SIGNAL(timeout()), this, SLOT(checkTitle()));
    connect(kadu, SIGNAL(changingStatus(UserStatus&, bool&)), this, SLOT(checkStatusDesc(UserStatus&, bool&)));
    connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
    connect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
    ChatList::ConstIterator it;
    for ( it = chat_manager->chats().begin(); it != chat_manager->chats().end(); it++ )
        handleCreatedChat(*it);

    // Configuration dialog
    ConfigDialog::registerSlotOnCreate(this, SLOT(onCreateConfig()));
    ConfigDialog::registerSlotOnClose(this, SLOT(onDestroyConfig()));
    ConfigDialog::registerSlotOnApply(this, SLOT(onUpdateConfig()));

    ConfigDialog::addTab("amaroK", dataPath("kadu/modules/data/amarok/amarok.png"));

        ConfigDialog::addVGroupBox("amaroK", "amaroK", QT_TRANSLATE_NOOP("@default", "Formats"));
            ConfigDialog::addLineEdit(config, "amaroK", "Formats", QT_TRANSLATE_NOOP("@default", "amaroK message format for chat windows"), "chatString", "amaroK: %t [%c / %l]",
                QT_TRANSLATE_NOOP("@default", "%t - song title, %f - file name, %l - song length (MM:SS),<br>%c - current song position (MM:SS), %p - percents of played song"));
            ConfigDialog::addLineEdit(config, "amaroK", "Formats", QT_TRANSLATE_NOOP("@default", "amaroK status format"), "statusString", "-=[%t]=-",
                QT_TRANSLATE_NOOP("@default", "%t - song title, %f - file name, %l - song length (MM:SS),<br>%c - current song position (MM:SS), %p - percents of played song"));

        ConfigDialog::addVGroupBox("amaroK", "amaroK", QT_TRANSLATE_NOOP("@default","Cut signatures"));
            ConfigDialog::addCheckBox(config, "amaroK", "Cut signatures", QT_TRANSLATE_NOOP("@default", "Enable signatures cutting"), "signature", TRUE, QT_TRANSLATE_NOOP("@default", "Shell I cut these craps?"));
            ConfigDialog::addTextEdit(config, "amaroK", "Cut signatures", QT_TRANSLATE_NOOP("@default", "Signatures:"), "signatures", DEFAULT_SIGNATURES);

        ConfigDialog::addCheckBox(config, "amaroK", "amaroK", QT_TRANSLATE_NOOP("@default","Put statuses switch to dock menu"), "dockMenu", false, QT_TRANSLATE_NOOP("@default","Enable this to put \"amaroK statuses switch\"\ninto docked icon menu."));
}

AmaroK::~AmaroK()
{
    kdebugf();
    int idx = dockMenu->indexOf(popups[2]);
    if (idx == -1)
        kadu->mainMenu()->removeItem(popups[2]);
    else
        dockMenu->removeItem(popups[2]);

    disconnect(timer, SIGNAL(timeout()), this, SLOT(checkTitle()));
    disconnect(kadu, SIGNAL(changingStatus(UserStatus&, bool&)), this, SLOT(checkStatusDesc(UserStatus&, bool&)));
    disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
    disconnect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
    ChatList::ConstIterator it;
    for ( it = chat_manager->chats().begin(); it != chat_manager->chats().end(); it++ )
        handleDestroyingChat(*it);

    ConfigDialog::unregisterSlotOnCreate(this, SLOT(onCreateConfig()));
    ConfigDialog::unregisterSlotOnClose(this, SLOT(onDestroyConfig()));
    ConfigDialog::unregisterSlotOnApply(this, SLOT(onUpdateConfig()));
    ConfigDialog::removeControl("amaroK", "Put statuses switch to dock menu");
    ConfigDialog::removeControl("amaroK", "Signatures:");
    ConfigDialog::removeControl("amaroK", "Enable signatures cutting");
    ConfigDialog::removeControl("amaroK", "Cut signatures");
    ConfigDialog::removeControl("amaroK", "amaroK message format for chat windows");
    ConfigDialog::removeControl("amaroK", "amaroK status format");
    ConfigDialog::removeControl("amaroK", "Formats");
    ConfigDialog::removeTab("amaroK");

    client->detach();

    delete client;
    delete menu;
    delete timer;
    delete config;
}

void AmaroK::handleCreatedChat(Chat* chat)
{
    kdebugf();

    QPushButton* amarok = new QPushButton(chat->buttontray);
    amarok->setPixmap(icons_manager.loadIcon("Unmute"));
    amarok->setPopup(menu);
    amarok->show();
    QToolTip::add(amarok, "amaroK");
    chatButtons[chat] = amarok;
}

void AmaroK::handleDestroyingChat(Chat* chat)
{
    kdebugf();

    delete chatButtons[chat];
    chatButtons.remove(chat);
}

void AmaroK::chatCreated(const UinsList& ul)
{
    kdebugf();
    Chat* chat = chat_manager->findChatByUins(ul);
    handleCreatedChat(chat);
}

void AmaroK::chatDestroying(const UinsList& ul)
{
    kdebugf();
    Chat* chat = chat_manager->findChatByUins(ul);
    handleDestroyingChat(chat);
}

bool AmaroK::isOn()
{
    kdebugf();
    QByteArray replyData, data;
    QCString replyType;
    if (client->call("amarok", "player", "isPlaying()", data, replyType, replyData))
        return true;
    else
        return false;
        
}

bool AmaroK::isPlaying()
{
    bool b;
    QByteArray replyData, data;
    QCString replyType;
    if (!client->call("amarok", "player", "isPlaying()", data, replyType, replyData))
        return false;

    QDataStream reply(replyData, IO_ReadOnly);
    reply >> b;
    return b;
}

QString AmaroK::getTitle()
{
    kdebugf();
    QByteArray replyData, data;
    QCString replyType;
    if (!client->call("amarok", "player", "nowPlaying()", data, replyType, replyData))
        return "";

    QDataStream reply(replyData, IO_ReadOnly);
    QString curTrack;
    reply >> curTrack;
    QString title( convert( curTrack.simplifyWhiteSpace() ) );

    // Now, if we want to cut nasty signatures, we do it!
    if (config->readBoolEntry("amaroK", "signature", TRUE))
    {
        QStringList sigList(QStringList::split('\n', config->readEntry("amaroK", "signatures", DEFAULT_SIGNATURES)));
        for (unsigned int i = 0; i < sigList.count(); i++)
            title.remove(sigList[i]);
    }

    return title;
}

int AmaroK::getLength()
{
    kdebugf();
    QByteArray replyData, data;
    QCString replyType;
    int t;
    if (!client->call("amarok", "player", "trackTotalTime()", data, replyType, replyData))
        return 0;

    QDataStream reply(replyData, IO_ReadOnly);
    reply >> t;
    return t;
}

int AmaroK::getCurrentPos()
{
    kdebugf();
    QByteArray replyData, data;
    QCString replyType;
    int pos;
    if (!client->call("amarok", "player", "trackCurrentTime()", data, replyType, replyData))
        return 0;

    QDataStream reply(replyData, IO_ReadOnly);
    reply >> pos;
    return pos;
}

void AmaroK::toggleTimer(int id)
{
    kdebugf();
    bool enabled;
    QPopupMenu* menu;
    int idx = dockMenu->indexOf(popups[2]);
    if (idx == -1)
        menu = kadu->mainMenu();
    else
        menu = dockMenu;

    enabled = menu->isItemChecked(popups[2]);

    if (!isOn() && !enabled)
    {
        MessageBox::msg(tr("amaroK isn't runned!"));
        return;
    }

    menu->setItemChecked(popups[2], !enabled);
    if (enabled)
        timer->stop();
    else
        timer->start(1000);
}

void AmaroK::checkTitle()
{
    if (!isOn())
    {
        timer->stop();
        kadu->mainMenu()->setItemChecked(popups[2], false);
        return;
    }
    if (!gadu->status().isOffline())
    {
        QString title = getTitle();
        if (title != currentTitle || !gadu->status().hasDescription())
        {
            currentTitle = title;
            gadu->status().setDescription(parse(config->readEntry("amaroK", "statusString")));
        }
    }
}

void AmaroK::putSongTitle(int ident)
{
    kdebugf();
    if (!isPlaying())
    {
        MessageBox::msg(tr("amaroK isn't playing!"));
        return;
    }
    Chat* chat = getCurrentChat();
    QString title;
    if (popups[0] == ident)
        title = parse(config_file.readEntry("amaroK", "chatString"));

    if (popups[1] == ident)
        title = getTitle();

    int x, y;

    HtmlDocument doc, doc2, doc3;
    chat->edit->getCursorPosition(&y, &x);
    chat->edit->insertAt(title, y, x);
    doc.parseHtml(chat->edit->text());
    for ( int i = 0; i < doc.countElements(); i++ )
    {
        if (i == 7)
        {
            doc2.parseHtml(doc.elementText(i));
            for ( int j = 0; j < doc2.countElements(); j++ )
            {
                if (doc2.isTagElement(j))
                    doc3.addTag(doc2.elementText(j));
                else
                    doc3.addText(doc2.elementText(j));
            }
        } else {
            if (doc.isTagElement(i))
                doc3.addTag(doc.elementText(i));
            else
                doc3.addText(doc.elementText(i));
        }
    }
    chat->edit->setText(doc3.generateHtml());
    chat->edit->moveCursor(QTextEdit::MoveEnd, FALSE);
}

QString AmaroK::parse(QString str)
{
    kdebugf();
    uint sl = str.length();
    QString r;

    for ( uint i = 0; i < sl; i++ )
    {
        while (str[i] != '%' && i < sl)
        {
            r += str[i];
            i++;
        }

        if (str[i] == '%')
        {
            i++;
            switch(str[i].latin1())
            {
                case 't':
                {
                    r += getTitle();
                    break;
                }
                case 'l':
                {
                    char ms[10];
                    int lgt = getLength(), m, s;
                    m = lgt / 60;
                    s = lgt % 60;
                    if (s < 10)
                        sprintf(ms, "%d:0%d", m, s);
                    else
                        sprintf(ms, "%d:%d", m, s);

                    r += ms;
                    break;
                }
                case 'c':
                {
                    char ms[10];
                    int lgt = getCurrentPos(), m, s;
                    m = lgt / 60;
                    s = lgt % 60;
                    if (s < 10)
                        sprintf(ms, "%d:0%d", m, s);
                    else
                        sprintf(ms, "%d:%d", m, s);

                    r += ms;
                    break;
                }
                case 'p':
                {
                    char tmp[10];
                    int perc = getCurrentPos() / (getLength()/1000);
                    sprintf(tmp, "%d%%", perc);
                    r += tmp;
                    break;
                }
                case 'd':
                {
                    r += descriptionToUse;
                    break;
                }
                default:
                    r += str[i];
            }
        }
    }
    return r;
}

Chat* AmaroK::getCurrentChat()
{
    kdebugf();
    ChatList cs = chat_manager->chats();
    uint i;
    for ( i = 0; i < cs.count(); i++ )
    {
        if (cs[i]->isActiveWindow())
        {
            break;
        }
    }
    if (i == cs.count())
        return 0;

    return cs[i];
}

void AmaroK::checkStatusDesc(UserStatus& status, bool &stop)
{
    QString str = config_file.readEntry("amaroK", "statusString");
    if (status.description().find("%amarok%") > -1)
    {
        QString newDesc = status.description().replace("%amarok%", parse(str));
        status.setDescription(newDesc);
        gadu->status().setStatus(status);
        stop = TRUE;
    }
}

QString AmaroK::convert(QString str)
{
    for (unsigned int i = 0; i < sizeof(CODE_CP1250); i++)
    {
        str.replace(CODE_CP1250[i],CODE_ISO8859_2[i]);
    }
    return QString::fromLocal8Bit(str,-1);
}

void AmaroK::onCreateConfig()
{
    modules_manager->moduleIncUsageCount("amarok");
}

void AmaroK::onDestroyConfig()
{
    onUpdateConfig();
    modules_manager->moduleDecUsageCount("amarok");
}

void AmaroK::onUpdateConfig()
{
    int idx = dockMenu->indexOf(popups[2]);
    if (idx == -1)
        kadu->mainMenu()->removeItem(popups[2]);
    else
        dockMenu->removeItem(popups[2]);

    bool enabled = false;
    if (timer->isActive())
        enabled = true;

    bool menuPos = config->readBoolEntry("amaroK", "dockMenu", false);
    if (menuPos)
    {
        popups[2] = dockMenu->insertItem(tr("Enable amaroK statuses"), this, SLOT(toggleTimer(int)), 0, -1, 10);
        dockMenu->setItemChecked(popups[2], enabled);
    }
    else
    {
        popups[2] = kadu->mainMenu()->insertItem(tr("Enable amaroK statuses"), this, SLOT(toggleTimer(int)), 0, -1, 0);
        kadu->mainMenu()->setItemChecked(popups[2], enabled);
    }

    config->sync();
}
